package com.pgpool.tests;


import com.pgpool.domain.AuthData;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Random;

public class Issue271_SingleThread extends PgpoolTestSupport {


	@Override
	public void runTest(AuthData authData) throws Exception {
		java.sql.DriverManager.setLogWriter(new java.io.PrintWriter(System.out));

		String url = String.format("jdbc:postgresql://%s:%d/%s?loglevel=2", authData.getPgpoolHostname(), authData.getPgpoolPort(), authData.getDatabaseName());

		Random random = new Random(System.currentTimeMillis());

		try (Connection conn = DriverManager.getConnection(url, authData.getDatabaseLogin(), authData.getDatabasePasswd())) {


			for (int j = 0; j < 50; j++) {

				conn.setAutoCommit(false); // here we define explicit transaction

				int testValue = random.nextInt();
				System.out.println("Test value " + testValue);

				try (PreparedStatement stmtInsert = conn.prepareStatement("select id from write_table where id = ?")) {

					for (int i = 0; i < 20; i++) {
						stmtInsert.setInt(1, testValue);
						try (ResultSet rs = stmtInsert.executeQuery()) {
							if (rs.next())
								throw new RuntimeException("This should not happen, please empty the 'write_table' and restart trhe test");
						}

					}
				}
				conn.commit();

				try (PreparedStatement stmtInsert = conn.prepareStatement("insert into write_table(id) values(?)")) {
					stmtInsert.setInt(1, testValue);
					stmtInsert.execute();
					System.out.println("Value inserted");
				}

				try (PreparedStatement stmtInsert = conn.prepareStatement("select id from write_table where id = ?")) {

					for (int i = 0; i < 20; i++) {
						stmtInsert.setInt(1, testValue);
						if (!stmtInsert.executeQuery().next()) {
							throw new RuntimeException("Balanced");
						}
					}
				}
				conn.commit();
			}
		}

	}
}
